/* USB Eye Catcher test program


                        VCC
                         |
                         R3
  USB D- ----+----- R1 --+- PD3
  USB D+ ----|--+-- R2 ---- PD2/INT0
             |  |
             D1 D2
	     |  |
         ----+--+---

    5V for Red/Green              4.4V for Red            
  USB 5V ---+----------> VCC    USB 5V ---+---|>|----> VCC
            C1                            C1              
  GND ------+                   GND ------+               



   +--------------> XTAL1
   |
   +---|[]|---+---> XTAL2
  C2          C3
   |          |
 --+----------+-- GND

 BU5004 Mapping

      SW1                                        SW2
      _                                           _
  +--o o--- ROW2 ---- PD4       PC0 ----- ROW1 --o o--+
 R4         COL4 -R6- PC5       PC1 -R13- COL8        R5
  |         COL3 -R7- PC4       PC2 ----- ROW3        |
 GND        COL2 -R8- PC3       PB1 ----- ROW4       GND
            ROW5 ---- PD5       PB5 -R12- COL7  
            ROW6 ---- PD6       PB4 -R11- COL6  
            COL1 -R9- PD7       PB3 -R10- COL5   
            ROW8 ---- PB0       PB2 ----- ROW7  

XTAL: 12.000 MHz
D1,D2  3V6

C1     10u
C2,C3  12-22p
R1,R2  75
R3     2K2
R4,R5  2K2
R6-13: 75 for Green or Red + 4.4V 
      100 for Red

           75 @ Green,5V          -> 35 mA 
           75 @ Red,(5 - 0.6V)    -> 36 mA 
          100 @ Red,5V            -> 33 mA 

BU5004R

    24 mm x 24 mm

         ROW2 1  o x x x x x o 16 ROW1
         COL4 2  o           o 15 COL8
         COL3 3  o           o 14 ROW3
         COL2 4  o           o 13 ROW4
         ROW5 5  o           o 12 COL7
         ROW6 6  o           o 11 COL6
         COL1 7  o           o 10 COL5
         ROW8 8  o           o  9 ROW7
                    (top view)

                             (K)
           +---------------+
           | | | | | | | | | ROW8
           +---------------+
           | | | | | | | | | ROW7
           +---------------+
                   :
           +---------------+
           | | | | | | | | | ROW1
           +---------------+
      COL   1 2 3 4 5 6 7 8
      (A)

*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <util/delay.h>

#include "usbcdc.h"
#include "buconf.h"

#define can_getc	usbcdc_can_getc
#define getc	usbcdc_getc
#define putc	usbcdc_putc
#define poll	usbcdc_poll

#define bit_set(port,bit)	{ port |= (1<<(bit));}
#define bit_clear(port,bit)	{ port &= ~(1<<(bit));}
#define min(a,b)		((a)<(b) ? (a):(b))

static void init_timeout() {
	TCCR1B = 3; // timer1 start @ clk/64
}

static inline uint16_t get_time() {
	return TCNT1;
}

#define TO_DOT		0
#define TO_FRAME	1
#define NR_TIMEOUT	2

static int8_t to_flag[NR_TIMEOUT];
static int16_t to_time[NR_TIMEOUT];

static void set_timeout(uint8_t chan, uint16_t t) {
	to_time[chan] = t + get_time();
	to_flag[chan] = 1;
}

static int8_t check_timeout(uint8_t chan) {
	if (to_flag[chan] && (to_time[chan] - (int16_t)(get_time()) <= 0)) {
		to_flag[chan] = 0;
	}
	return !to_flag[chan];
}

static void __delay(uint8_t chan, uint16_t t) {
	set_timeout(chan, t);
	while (check_timeout(chan)) {
		wdt_reset();
		poll();
	}
}

static inline void port_setup() {
	bit_set(COL1_DDR,COL1);
	bit_set(COL2_DDR,COL2);
	bit_set(COL3_DDR,COL3);
	bit_set(COL4_DDR,COL4);
	bit_set(COL5_DDR,COL5);
	bit_set(COL6_DDR,COL6);
	bit_set(COL7_DDR,COL7);
	bit_set(COL8_DDR,COL8);
	bit_clear(ROW1_PORT,ROW1);
	bit_clear(ROW2_PORT,ROW2);
	bit_clear(ROW3_PORT,ROW3);
	bit_clear(ROW4_PORT,ROW4);
	bit_clear(ROW5_PORT,ROW5);
	bit_clear(ROW6_PORT,ROW6);
	bit_clear(ROW7_PORT,ROW7);
	bit_clear(ROW8_PORT,ROW8);
}

static int8_t cur_col,cur_row;

static void disp_off() {
	switch(cur_row) {
	case 0: bit_clear(COL1_PORT,COL1); break;
	case 1: bit_clear(COL2_PORT,COL2); break;
	case 2: bit_clear(COL3_PORT,COL3); break;
	case 3: bit_clear(COL4_PORT,COL4); break;
	case 4: bit_clear(COL5_PORT,COL5); break;
	case 5: bit_clear(COL6_PORT,COL6); break;
	case 6: bit_clear(COL7_PORT,COL7); break;
	case 7: bit_clear(COL8_PORT,COL8); break;
	}
	switch(cur_col) {
	case 0:  bit_clear(ROW1_DDR, ROW1); break;
	case 1:  bit_clear(ROW2_DDR, ROW2); break;
	case 2:  bit_clear(ROW3_DDR, ROW3); break;
	case 3:  bit_clear(ROW4_DDR, ROW4); break;
	case 4:  bit_clear(ROW5_DDR, ROW5); break;
	case 5:  bit_clear(ROW6_DDR, ROW6); break;
	case 6:  bit_clear(ROW7_DDR, ROW7); break;
	case 7:  bit_clear(ROW8_DDR, ROW8); break;
	}
}

static void disp_on(int8_t row,int8_t col) {
	switch(row) {
	case 0: bit_set(COL1_PORT,COL1); break;
	case 1: bit_set(COL2_PORT,COL2); break;
	case 2: bit_set(COL3_PORT,COL3); break;
	case 3: bit_set(COL4_PORT,COL4); break;
	case 4: bit_set(COL5_PORT,COL5); break;
	case 5: bit_set(COL6_PORT,COL6); break;
	case 6: bit_set(COL7_PORT,COL7); break;
	case 7: bit_set(COL8_PORT,COL8); break;
	}
	switch(col) {
	case 0:  bit_set(ROW1_DDR, ROW1); break;
	case 1:  bit_set(ROW2_DDR, ROW2); break;
	case 2:  bit_set(ROW3_DDR, ROW3); break;
	case 3:  bit_set(ROW4_DDR, ROW4); break;
	case 4:  bit_set(ROW5_DDR, ROW5); break;
	case 5:  bit_set(ROW6_DDR, ROW6); break;
	case 6:  bit_set(ROW7_DDR, ROW7); break;
	case 7:  bit_set(ROW8_DDR, ROW8); break;
	}
	cur_row = row;
	cur_col = col;
}

static uint8_t button_read() {
	static uint8_t button_state;
	static uint8_t b1_state;
	static uint8_t b2_state;
	uint8_t tmp;

#ifdef SW_USE_COL
	bit_clear(COL4_DDR,COL4);
	bit_clear(COL8_DDR,COL8);
	bit_set(COL4_PORT,COL4);
	bit_set(COL8_PORT,COL8);
	b1_state <<=1;
	b2_state <<=1;
	_delay_loop_1(30);
	if (bit_is_clear(COL4_PIN, COL4)) {
		b1_state |= 1;
	}
	if (bit_is_clear(COL8_PIN, COL8)) {
		b2_state |= 1;
	}
	bit_clear(COL4_PORT,COL4);
	bit_clear(COL8_PORT,COL8);
	bit_set(COL4_DDR,COL4);
	bit_set(COL8_DDR,COL8);
#else
	bit_set(ROW2_PORT,ROW2);
	bit_set(ROW1_PORT,ROW1);
	b1_state <<=1;
	b2_state <<=1;
	_delay_loop_1(30);
	if (bit_is_clear(ROW2_PIN, ROW2)) {
		b1_state |= 1;
	}
	if (bit_is_clear(ROW1_PIN, ROW1)) {
		b2_state |= 1;
	}
	bit_clear(ROW1_PORT,ROW1);
	bit_clear(ROW2_PORT,ROW2);
#endif
	if ( (b1_state == 0 || b1_state == 0xff) && 
	     (b2_state == 0 || b2_state == 0xff) ) {
		tmp = (b1_state & 1) | (b2_state & 2);
		if (tmp != button_state) {
			putc('0'+tmp);
			putc('\r');
			putc('\n');
		}
		button_state = tmp;
	}
	return button_state;
}


int main(void)
{
    uint8_t ch;
    uint8_t x,y;
    uint8_t frame;
    uint8_t mode = 3;

#ifndef NO_CLKPR_SET
    /* set system_clock pre scaler 1/8 => 1/1 */
    CLKPR = (1<<CLKPCE);
    CLKPR = 0;
#endif

    wdt_enable(WDTO_1S);
    usbcdc_init();
    init_timeout();
    port_setup();

    sei();

    x = 0; y = 0;
    frame = 0;
    for(;;) {
	wdt_reset();
	poll();

	if (can_getc(1)) {
		ch = getc();
		if (ch == 'S') { // get Software
			putc('E');
			putc('Y');
			putc('E');
			putc('C');
			putc('A');
			putc('T');
			putc(' ');
		} else if (ch == 'z') {
			mode = 0;
		} else if (ch == 'x') {
			mode = 1;
		} else if (ch == 'c') {
			mode = 2;
		} else if (ch == 'v') {
			mode = 3;
		}
	}
	if(check_timeout(TO_DOT)) {
		uint8_t on = 0;
		uint8_t tmp;

		x++;
		if (x >= 8) {
			x = 0;
			y ++;
			if (y >= 8) {
				y = 0;
			}
		}
		disp_off();
		if (x == 0 && y == 0) {
			tmp = button_read();
			if (tmp == 1) {
				mode = 1;
			} else if (tmp == 2) {
				mode = 2;
			}
		}
		if (mode == 0) {
			on = 1;
		} else if (mode == 1) {
			if (x == (frame>>3)) on = 1;
		} else if (mode == 2) {
			if (y == (frame>>3)) on = 1;
		} else {
			on = 0;
		}
		if (on) disp_on(x,y);
    		set_timeout(TO_DOT, 24);
	}
	if(check_timeout(TO_FRAME)) {
		frame++;
		if (frame >= 64) {
			frame = 0;
		}
		set_timeout(TO_FRAME, 11718); // 12000000 / 64 / 16 (16Hz)
	}
    }
    return 0;
}

